# -*- coding: utf-8 -*-
# Python version: 2/3
#
# Dataset loader for NLP experiments.
# Simon Fraser University
# Jetic Gu
#
#
import os
import sys
import inspect
import unittest
import importlib

from .format import *

__version__ = "0.3a"

supportedList = {
    'tree': tree,
    'txtFiles': txtFiles,
    'txt': txt,
    'AMR': AMR,
    'txtOrTree': txtOrTree,
    'pyCode': pyCode
}


class ParallelDataLoader():
    def __init__(self, srcFormat='txtOrTree', tgtFormat='txtOrTree'):
        self.srcLoader = supportedList[srcFormat].load
        #    importlib.import_module(srcFormat, format).load
        self.tgtLoader = supportedList[tgtFormat].load
        return

    def load(self, fFile, eFile, linesToLoad=sys.maxsize):
        data = zip(self.srcLoader(fFile, linesToLoad),
                   self.tgtLoader(eFile, linesToLoad))
        # Remove incomplete or invalid entries
        data = [(f, e) for f, e in data if f is not None and e is not None]
        data = [(f, e) for f, e in data if len(f) > 0 and len(e) > 0]
        return data



# -*- coding: utf-8 -*-
# Python version: 2/3
#
# Abstract Meaning Representation Graph class
# Simon Fraser University
# Jetic Gu
#
# This module contains functions and classes necessary for loading Abstract
# Meaning Representation trees. 2 examples are provided in TestTree.
#
import sys
import os
import unittest
import inspect
from copy import deepcopy
from six import string_types


class NodeAMR:
    '''
    This is the main data structure of a tree, a Node instance is a node on the
    tree. The structure of the subtree with node x as root can be viewed by
    calling x.__repr__()
    '''
    def __init__(self, hyperlink=None):
        self.id = ""  # instance id: "b" in "(b/boy)"
        self.concept = ""  # concept name: "boy" in "(b/boy)"
        self.hyperlink = None
        self.parent = None
        self.link = []  # arguments
        self.linkType = ""
        if hyperlink is not None:
            self.id = hyperlink.id
            self.hyperlink = hyperlink
        return

    def __repr__(self, __spacing="", __dispChild=True):
        '''
        This method prints the structure of the graph.
        '''
        if __dispChild is False:
            return "NodeAMR"
        if self.hyperlink is None:
            sys.stdout.write("(" + self.id + " / " + self.concept)
            for (relation, entry) in self.link:
                sys.stdout.write("\n" + __spacing + "      " + relation + " ")
                if isinstance(entry, NodeAMR):
                    entry.__repr__(__spacing + "      ")
                else:
                    sys.stdout.write(entry)
            sys.stdout.write(")")
            if __spacing == "":
                sys.stdout.write("\n")
        else:
            sys.stdout.write(self.id)
        return "NodeAMR"

    def __len__(self):
        raise NotImplemented

    def export(self):
        if self.hyperlink is not None:
            return self.id

        result = "( " + self.id + " / " + self.concept
        for (relation, entry) in self.link:
            result += " " + relation + " "
            if isinstance(entry, NodeAMR):
                result += entry.export()
            else:
                result += entry
        result += " )"
        return result


def constructAMRFromStr(string):
    '''
    This method constructs an AMR graph from a string.
    @param string: str, in Penn Treebank format
    @return root: NodeAMR, the root node.
    '''
    elements = string.replace(" / ", "-/-").split("\"")
    for i in range(len(elements)):
        if i % 2 == 0:
            elements[i] = elements[i].replace(")", "\t)\t")
            elements[i] = elements[i].replace("(", "\t(\t")
            elements[i] = elements[i].replace(" ", "\t")

    elements = [i for i in "\"".join(elements).split("\t") if i != ""]
    instances = {}

    # Generate Instances
    for i in range(len(elements)):
        if "-/-" in elements[i]:
            newInstance = NodeAMR()
            newInstance.id, newInstance.concept =\
                tuple(elements[i].split("-/-"))
            instances[newInstance.id] = newInstance
            elements[i] = newInstance
        elif i != 0 and isinstance(elements[i - 1], string_types) and\
                elements[i - 1][0] == ":" and elements[i] in instances:
            newInstance = NodeAMR(hyperlink=instances[elements[i]])
            elements[i] = newInstance

    def constructGraph(elements):
        result = None
        i = -1
        count = 0
        main = None
        while i < len(elements) - 1:
            i += 1

            if isinstance(elements[i], NodeAMR) and\
                    elements[i].hyperlink is None:
                main = elements[i]
                continue
            if isinstance(elements[i], NodeAMR) and\
                    elements[i].hyperlink is not None:
                main.link[-1] += (elements[i],)
                continue
            if elements[i] == "(":
                count += 1
                if count > 1:
                    length, Node = constructGraph(elements[i:])
                    main.link[-1] += (Node,)
                    i += length - 1
                continue
            if elements[i] == ")":
                count -= 1
                if count == 0:
                    return i, main
                continue
            if elements[i][0] == ":":
                main.link.append((elements[i], ))
                continue
            if elements[i][0] == "\"":
                main.link[-1] += (elements[i],)
                continue
            main.link[-1] += (elements[i],)

    _, root = constructGraph(elements)
    return root


def load(fileName, linesToLoad=sys.maxsize, verbose=False):
    import progressbar
    fileName = os.path.expanduser(fileName)
    content = []
    i = 0
    widgets = [progressbar.Bar('>'), ' ', progressbar.ETA(),
               progressbar.FormatLabel(
               '; Total: %(value)d sents (in: %(elapsed)s)')]
    if verbose is False:
        loadProgressBar =\
            progressbar.ProgressBar(widgets=widgets,
                                    maxval=min(
                                        sum(1 for line in open(fileName)),
                                        linesToLoad)).start()
    for line in open(fileName):
        i += 1
        if verbose is False:
            loadProgressBar.update(i)
        content.append(constructAMRFromStr(line))
        if i == linesToLoad:
            break
    if verbose is False:
        loadProgressBar.finish()
    return content


class TestAMR(unittest.TestCase):
    def testBuildAMR_A(self, x=None):
        str = "( a / and :op1 ( i / international ) :op2 ( m / military ) " +\
            ":op3 ( t / terrorism ) )"
        if x is None:
            x = constructAMRFromStr(str)
        self.assertEqual(str.split(), x.export().split())
        return

    def testBuildAMR_B(self, x=None):
        str =\
            "( s / start-01 :ARG0 ( p / picture :ARG1-of ( l / " +\
            "look-forward-03 :ARG0 ( p2 / person ) :ARG1-of ( l2 / long-03 " +\
            ") ) ) :ARG1 ( e / emerge-01 :ARG1-of ( f / frequent-02 ) :loca" +\
            "tion ( m / media :mod ( v / various ) :ARG1-of ( m2 / major-02" +\
            " ) :location ( c / city :wiki \"Hong_Kong\" :name ( n / name :" +\
            "op1 \"Hong\" :op2 \"Kong\" ) ) ) ) :time ( d / date-entity :ye" +\
            "ar 2005 :season ( s2 / summer ) ) )"
        if x is None:
            x = constructAMRFromStr(str)
        self.assertEqual(str.split(), x.export().split())
        return

    def testBuildAMR_C(self, x=None):
        str =\
            "(w / want-01 :ARG0 (b / boy) :ARG1 (g / go-02 :ARG0 b))"
        if x is None:
            x = constructAMRFromStr(str)
        str = str.replace("(", " ( ").replace(")", " ) ")
        self.assertEqual(str.split(), x.export().split())
        return

    def testLoadAMRFromFile(self):
        import warnings
        currentdir = os.path.dirname(
            os.path.abspath(inspect.getfile(inspect.currentframe())))
        parentdir = os.path.dirname(currentdir)
        warnings.simplefilter("ignore")
        content = load(parentdir + "/test/sampleAMR.amr", verbose=True)
        rawText = list(open(parentdir + "/test/sampleAMR.amr"))
        for amr, str in zip(content, rawText):
            self.assertEqual(str.split(), amr.export().split())
        return


if __name__ == '__main__':
    import __main__ as main
    if not bool(getattr(sys, 'ps1', sys.flags.interactive)):
        unittest.main()
    else:
        str = "( a / and :op1 ( i / international ) :op2 ( m / military ) " +\
            ":op3 ( t / terrorism ) )"
        x = constructAMRFromStr(str)
        str =\
            "( s / start-01 :ARG0 ( p / picture :ARG1-of ( l / " +\
            "look-forward-03 :ARG0 ( p2 / person ) :ARG1-of ( l2 / long-03 " +\
            ") ) ) :ARG1 ( e / emerge-01 :ARG1-of ( f / frequent-02 ) :loca" +\
            "tion ( m / media :mod ( v / various ) :ARG1-of ( m2 / major-02" +\
            " ) :location ( c / city :wiki \"Hong_Kong\" :name ( n / name :" +\
            "op1 \"Hong\" :op2 \"Kong\" ) ) ) ) :time ( d / date-entity :ye" +\
            "ar 2005 :season ( s2 / summer ) ) )"
        y = constructAMRFromStr(str)
        str =\
            "(w / want-01 :ARG0 (b / boy) :ARG1 (g / go-02 :ARG0 b))"
        z = constructAMRFromStr(str)
        print("Use the three Nodes x, y, z for testing new methods on Node.")
        print("Use unittest.main() to start unit test")



# -*- coding: utf-8 -*-
# Python version: 2.7
#
# Dependency Tree class
# Simon Fraser University
# Ruoyi Wang
#
# This module contains is used for loading
# penn treebank format sentences with dependency information
# 2 examples are provided in sampleDepTree.txt
#

import os
import sys
import inspect
from bisect import bisect_left
import itertools
import copy
import unittest

from .semanticFrame import load as loadSemFrame

FORM_OFFSET = 1
PPOS_OFFSET = 7
HEAD_OFFSET = 8
DEPREL_OFFSET = 9
PRED_OFFSET = 10
ARGS_OFFSET = 11


class TreeNode:
    def __init__(self, parent=None):
        # value = (POS, FORM)
        self.value = None
        self.parent = parent
        # dependency relationship with parent
        self.rel = None
        # frame is the predicate name
        self.frame = None
        # args: {arg name: arg node}
        self.args = None

        # next sibling
        self.next_sib = None
        # first left child
        self.flc = None
        # first right child
        self.frc = None

        self.phrase = []

    def last_left_child(self):
        if self.flc is None:
            return None
        node = self.flc
        while node.next_sib is not None:
            node = node.next_sib
        return node

    def last_right_child(self):
        if self.frc is None:
            return None
        node = self.frc
        while node.next_sib is not None:
            node = node.next_sib
        return node

    def append_left_child(self, node, rel):
        node.parent = self
        node.rel = rel
        llc = self.last_left_child()
        if llc is None:
            self.flc = node
        else:
            llc.next_sib = node

    def append_right_child(self, node, rel):
        node.parent = self
        node.rel = rel
        lrc = self.last_right_child()
        if lrc is None:
            self.frc = node
        else:
            lrc.next_sib = node

    def __repr__(self):
        return 'TreeNode({})'.format(self.value)

    def calcPhrase(self, force=False):  # todo: figure out the desired result
        raise NotImplementedError


def export_to_vec(node):
    """
    export the tree at node to a vec of (POS, FORM) representing the original
    sentence
    """
    words = []
    for n in inorder_traversal(node):
        words.append(n.info)
    return words


def export_to_table(node):
    """
    export the tree to a table
    columns:
    0: id. starting from 1
    1: FORM
    2: POS
    3: parent id
    4: dependency relationship
    5: predicate name
    6+: arguments info
    """
    # reconstruct args_data
    args_data = []
    for n in inorder_traversal(node):
        if n.frame is not None:
            args_data.append((n, n.frame, n.args))

    table = []
    indices = {}
    for i, n in enumerate(inorder_traversal(node)):
        table.append([i + 1, n.info[1], n.info[0]])
        indices[n] = i

    for n, i in indices.items():
        if n.parent is None:
            table[i].extend([0, 'ROOT'])
        else:
            table[i].extend([indices[n.parent] + 1, n.rel])

    for x in table:
        x.append('_')
    for arg_data in args_data:
        pred_node, pred_name, args_dict = arg_data
        table[indices[pred_node]][-1] = pred_name

    for arg_data in args_data:
        for x in table:
            x.append('_')
        pred_node, pred_name, args_dict = arg_data
        for arg_name, arg_node in args_dict.items():
            table[indices[arg_node]][-1] = arg_name

    return table


def inorder_traversal(node):
    if node.flc is not None:
        for n in inorder_traversal(node.flc):
            yield n
    yield node
    if node.frc is not None:
        for n in inorder_traversal(node.frc):
            yield n
    if node.next_sib is not None:
        for n in inorder_traversal(node.next_sib):
            yield n


def _is_pred(word):
    return word[PRED_OFFSET] != '_'


def read_sentences(f):
    """split the file into tables, each representing a sentence"""
    lines = []
    for line in f:
        if line == '\n':
            yield lines
            del lines[:]
        else:
            lines.append(line.strip().split('\t'))
    if len(lines):
        yield lines


def parse_sentence(pb_names, sentence):
    """construct the dependency tree and return the root"""
    nodes = []
    # construct a node for each word
    for word in sentence:
        node = TreeNode()
        node.value = word[PPOS_OFFSET], word[FORM_OFFSET]
        nodes.append(node)

    # link the nodes into a tree
    root = None
    for i, word in enumerate(sentence):
        head_idx, rel = int(word[HEAD_OFFSET]), word[DEPREL_OFFSET]
        if head_idx > 0:
            parent_idx = head_idx - 1
            if parent_idx > i:
                nodes[parent_idx].append_left_child(nodes[i], rel)
            else:
                nodes[parent_idx].append_right_child(nodes[i], rel)
        else:
            root = nodes[i]

    # extract predicate info
    preds = []
    pred_names = []
    for i, word in enumerate(sentence):
        if _is_pred(word):
            pred_name = word[PRED_OFFSET]
            preds.append(nodes[i])
            pred_names.append(pred_name)

    # link predicate info to the nodes
    args_data = [(x, y, {}) for x, y in zip(preds, pred_names)]
    for i, word in enumerate(sentence):
        args = word[ARGS_OFFSET:]
        for j, arg in enumerate(args):
            if arg != '_':
                args_data[j][-1][arg] = nodes[i]

    # filter irrelevant predicates
    pb_data = _filter_args_data(pb_names, args_data)

    # insert predicates info into the tree nodes
    for node, name, d in pb_data:
        node.frame = name
        node.args = d
    return root


def _load_frames(frames_path):
    """load frame info"""
    pb_frames = loadSemFrame('{}/*.xml'.format(frames_path))
    return pb_frames


def _process_frames(frames):
    """extract only the frame name and sort"""
    result = [x[0] for x in frames]
    result.sort()
    return result


def _verify_name(names, pred_name):
    """verify the pred name is in the SORTED list of pred names"""
    idx = bisect_left(names, pred_name)
    return idx < len(names) and names[idx] == pred_name


def _filter_args_data(pb_names, args_data):
    """remove args data that are not in TreeBank"""
    pb_data = []
    for data in args_data:
        pred_name = data[1]
        if _verify_name(pb_names, pred_name):
            pb_data.append(data)

    return pb_data


def parse_dep_tree(frames_path, f):
    """
    main entry point
    read dependency tree file and construct a forest
    each tree represents a sentence
    trees appear in the forest in the order of the original sentences
    """
    sentences = read_sentences(f)
    pb_frames = _load_frames(frames_path)
    pb_names = _process_frames(pb_frames)

    forest = []
    for sentence in sentences:
        root = parse_sentence(pb_names, sentence)
        forest.append(root)
    return forest


def read_back_sentence(sentence):
    nodes = []
    # construct a node for each word
    for word in sentence:
        node = TreeNode()
        node.value = word[2], word[1]
        nodes.append(node)

    # link the nodes into a tree
    root = None
    for i, word in enumerate(sentence):
        head_idx, rel = int(word[3]), word[4]
        if head_idx > 0:
            parent_idx = head_idx - 1
            if parent_idx > i:
                nodes[parent_idx].append_left_child(nodes[i], rel)
            else:
                nodes[parent_idx].append_right_child(nodes[i], rel)
        else:
            root = nodes[i]

    preds = []
    for i, word in enumerate(sentence):
        if word[5] != '_':
            pred_name = word[5]
            preds.append(nodes[i])
            nodes[i].frame = pred_name

    for n in preds:
        n.args = {}

    for i, word in enumerate(sentence):
        args = word[6:]
        for j, arg in enumerate(args):
            if arg != '_':
                preds[j].args[arg] = nodes[i]

    return root


def level_order_traversal(root):
    """
    :returns
    1st iteration: [[[root]]]
    following iterations:
    [[[left children], [right children]], [[], []], .. for each node from the
    previous level]
    """
    level = [[[root]]]
    while level:
        yield level
        cache = copy.copy(level)
        level = []
        for nodes in cache:
            for node in itertools.chain.from_iterable(nodes):
                x = node.flc
                lc = []  # left children
                while x is not None:
                    lc.append(x)
                    x = x.next_sib
                x = node.frc
                rc = []  # right children
                while x is not None:
                    rc.append(x)
                    x = x.next_sib
                node_children = [lc, rc]
                level.append(node_children)


def get_column_format(root):
    parent_id = 0
    par_column = []
    sib_column = []
    val_column = []
    has_left_child = []
    has_right_child = []
    has_sib = []
    for level in level_order_traversal(root):
        for branches in level:  # for each 2 branches from the previous node
            for branch in branches:
                for n in branch:
                    # parent id
                    par_column.append(parent_id)
                    # next sib exist?
                    if n.next_sib is not None:
                        sib_column.append(1)
                    else:
                        sib_column.append(0)
                    # value
                    val_column.append(n.info)
                    # has left child?
                    if n.flc is not None:
                        has_left_child.append(1)
                    else:
                        has_left_child.append(0)
                    # has right child?
                    if n.frc is not None:
                        has_right_child.append(1)
                    else:
                        has_right_child.append(0)
                    # has sib?
                    if len(branch) > 1:
                        has_sib.append(1)
                    else:
                        has_sib.append(0)
            parent_id += 1
    return (par_column, sib_column, val_column,
            has_left_child, has_right_child, has_sib)


def load(fileName, linesToLoad=sys.maxsize, verbose=False):
    # todo: requires pb_frames as additional information
    raise NotImplementedError


# unit test
class TestParse(unittest.TestCase):
    def setUp(self):
        with open(parentdir + '/test/sampleDepTree.txt') as file:
            self.forest = parse_dep_tree('../../../../data/pb_frames', file)
            # todo: add pb_frames to sample files

    def test_parse(self):
        correct_0 = [[1, 'Ms.', 'NNP', 2, 'TITLE', '_', '_'],
                     [2, 'Haag', 'NNP', 3, 'SBJ', '_', 'A0'],
                     [3, 'plays', 'VBZ', 0, 'ROOT', 'play.02', '_'],
                     [4, 'Elianti', 'NNP', 3, 'OBJ', '_', 'A1'],
                     [5, '.', '.', 3, 'P', '_', '_']]

        correct_1 = [[1, 'Bell', 'NNP', 8, 'SBJ', '_', 'A1', 'A0', 'A0'],
                     [2, ',', ',', 1, 'P', '_', '_', '_', '_'],
                     [3, 'based', 'VBN', 1, 'APPO', 'base.01', '_', '_', '_'],
                     [4, 'in', 'IN', 3, 'LOC', '_', 'AM-LOC', '_', '_'],
                     [5, 'Los', 'NNP', 6, 'NAME', '_', '_', '_', '_'],
                     [6, 'Angeles', 'NNP', 4, 'PMOD', '_', '_', '_', '_'],
                     [7, ',', ',', 1, 'P', '_', '_', '_', '_'],
                     [8, 'makes', 'VBZ', 0, 'ROOT', 'make.01', '_', '_', '_'],
                     [9, 'and', 'CC', 8, 'COORD', '_', '_', '_', '_'],
                     [10, 'distributes', 'VBZ', 9, 'CONJ', 'distribute.01',
                      '_', '_', '_'],
                     [11, 'electronic', 'JJ', 16, 'NMOD', '_', '_', '_', '_'],
                     [12, ',', ',', 11, 'P', '_', '_', '_', '_'],
                     [13, 'computer', 'NN', 11, 'COORD', '_', '_', '_', '_'],
                     [14, 'and', 'CC', 13, 'COORD', '_', '_', '_', '_'],
                     [15, 'building', 'NN', 14, 'CONJ', '_', '_', '_', '_'],
                     [16, 'products', 'NNS', 8, 'OBJ', '_', '_', 'A1', 'A1'],
                     [17, '.', '.', 8, 'P', '_', '_', '_', '_']]
        self.assertEqual(export_to_table(self.forest[0]), correct_0)
        self.assertEqual(export_to_table(self.forest[1]), correct_1)

    def test_read_back(self):
        tree = self.forest[1]
        original_dump = export_to_table(tree)
        new_tree = read_back_sentence(original_dump)
        new_dump = export_to_table(new_tree)
        self.assertEqual(original_dump, new_dump)


class TestColumnFormat(unittest.TestCase):
    def test_level_order_traversal(self):
        sentence = [[1, 'Ms.', 'NNP', 2, 'TITLE', '_', '_'],
                    [2, 'Haag', 'NNP', 3, 'SBJ', '_', 'A0'],
                    [3, 'plays', 'VBZ', 0, 'ROOT', 'play.02', '_'],
                    [4, 'Elianti', 'NNP', 3, 'OBJ', '_', 'A1'],
                    [5, '.', '.', 3, 'P', '_', '_']]
        root = read_back_sentence(sentence)
        self.assertEqual([
            [[[root]]],
            [[[root.flc], [root.frc, root.frc.next_sib]]],
            [[[root.flc.flc], []], [[], []], [[], []]],
            [[[], []]]
        ],
            list(level_order_traversal(root)))

    def test_column_format(self):
        sentence = [[1, 'Ms.', 'NNP', 2, 'TITLE', '_', '_'],
                    [2, 'Haag', 'NNP', 3, 'SBJ', '_', 'A0'],
                    [3, 'plays', 'VBZ', 0, 'ROOT', 'play.02', '_'],
                    [4, 'Elianti', 'NNP', 3, 'OBJ', '_', 'A1'],
                    [5, '.', '.', 3, 'P', '_', '_']]
        root = read_back_sentence(sentence)
        self.assertEqual(([0, 1, 1, 1, 2],
                          [0, 0, 1, 0, 0],
                          [('VBZ', 'plays'), ('NNP', 'Haag'),
                           ('NNP', 'Elianti'), ('.', '.'), ('NNP', 'Ms.')],
                          [1, 1, 0, 0, 0],
                          [1, 0, 0, 0, 0],
                          [0, 0, 1, 1, 0]),
                         get_column_format(root))

    def test_column_format_complex(self):
        sentence = [[1, 'Bell', 'NNP', 8, 'SBJ', '_', 'A1', 'A0', 'A0'],
                    [2, ',', ',', 1, 'P', '_', '_', '_', '_'],
                    [3, 'based', 'VBN', 1, 'APPO', 'base.01', '_', '_', '_'],
                    [4, 'in', 'IN', 3, 'LOC', '_', 'AM-LOC', '_', '_'],
                    [5, 'Los', 'NNP', 6, 'NAME', '_', '_', '_', '_'],
                    [6, 'Angeles', 'NNP', 4, 'PMOD', '_', '_', '_', '_'],
                    [7, ',', ',', 1, 'P', '_', '_', '_', '_'],
                    [8, 'makes', 'VBZ', 0, 'ROOT', 'make.01', '_', '_', '_'],
                    [9, 'and', 'CC', 8, 'COORD', '_', '_', '_', '_'],
                    [10, 'distributes', 'VBZ', 9, 'CONJ', 'distribute.01', '_',
                     '_', '_'],
                    [11, 'electronic', 'JJ', 16, 'NMOD', '_', '_', '_', '_'],
                    [12, ',', ',', 11, 'P', '_', '_', '_', '_'],
                    [13, 'computer', 'NN', 11, 'COORD', '_', '_', '_', '_'],
                    [14, 'and', 'CC', 13, 'COORD', '_', '_', '_', '_'],
                    [15, 'building', 'NN', 14, 'CONJ', '_', '_', '_', '_'],
                    [16, 'products', 'NNS', 8, 'OBJ', '_', '_', 'A1', 'A1'],
                    [17, '.', '.', 8, 'P', '_', '_', '_', '_']]
        root = read_back_sentence(sentence)
        self.assertEqual(([0, 1, 1, 1, 1, 2, 2, 2, 3, 4, 7, 10, 10, 11, 13, 14,
                           15],
                          [0, 0, 1, 1, 0, 1, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0],
                          [('VBZ', 'makes'), ('NNP', 'Bell'), ('CC', 'and'),
                           ('NNS', 'products'), ('.', '.'),
                           (',', ','), ('VBN', 'based'), (',', ','),
                           ('VBZ', 'distributes'), ('JJ', 'electronic'),
                           ('IN', 'in'), (',', ','), ('NN', 'computer'),
                           ('NNP', 'Angeles'), ('CC', 'and'),
                           ('NNP', 'Los'), ('NN', 'building')],
                          [1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0],
                          [1, 1, 1, 0, 0, 0, 1, 0, 0, 1, 1, 0, 1, 0, 1, 0, 0],
                          [0, 0, 1, 1, 1, 1, 1, 1, 0, 0, 0, 1, 1, 0, 0, 0, 0]),
                         get_column_format(root))


if __name__ == '__main__':
    if not bool(getattr(sys, 'ps1', sys.flags.interactive)):
        unittest.main()
    else:
        sentence = [[1, 'Ms.', 'NNP', 2, 'TITLE', '_', '_'],
                    [2, 'Haag', 'NNP', 3, 'SBJ', '_', 'A0'],
                    [3, 'plays', 'VBZ', 0, 'ROOT', 'play.02', '_'],
                    [4, 'Elianti', 'NNP', 3, 'OBJ', '_', 'A1'],
                    [5, '.', '.', 3, 'P', '_', '_']]
        root = read_back_sentence(sentence)
        x = get_column_format(root)



# -*- coding: utf-8 -*-
# Python version: 2/3
#
# Txt file format loader
# Simon Fraser University
# Jetic Gu
#
#
import os
import sys
import inspect
import unittest
__version__ = "0.3a"


def load(files, linesToLoad=sys.maxsize):
    '''
    This function is used to read a set of files with different information
    (e.g. POS, Form, etc.) on the same set of tokens.

    @param files: list of str, the files include FORM, POS, etc.,
    @param* linesToLoad: int, the lines to read
    @return: list of sentences. Each sentence is a list of tuples with POS,
        FORM, etc.
    '''
    content =\
        [list(zip(*[content.strip().split() for content in contents]))
         for contents in list(zip(*[open(os.path.expanduser(f))
                                    for f in files]))[:linesToLoad]]
    return content



# -*- coding: utf-8 -*-
# Python version: 2/3
#
# CoNLL U data format
# Simon Fraser University
# Jetic Gu
#
from __future__ import print_function

import os
import sys
import copy
import inspect
import unittest
import progressbar


defaultEntryIndex = {
    # This is taken from http://universaldependencies.org/format.html
    # CoNLL-U
    "ID": 0,  # Word index, integer starting at 1 for each new sentence;
    "FORM": 1,  # Word form or punctuation symbol.
    "LEMMA": 2,  # Lemma or stem of word form.
    "UPOS": 3,  # Universal part-of-speech tag.
    "XPOS": 4,  # Language-specific part-of-speech tag; underscore if not
                # available.
    "FEATS": 5,  # List of morphological features from the universal feature
                 # inventory or from a defined language-specific extension;
                 # underscore if not available.
    "HEAD": 6,  # Head of the current word, which is either a value of ID or
                # zero (0).
    "DEPREL": 7,  # Universal dependency relation to the HEAD (root iff HEAD =
                  # 0) or a defined language-specific subtype of one.
    "DEPS": 8,  # Enhanced dependency graph in the form of a list of
                # head-deprel pairs.
                # Note (Jetic): Universal dependencies don't use this at all.
    "MISC": 9,  # Any other annotation.
    "__name__": "CoNLL U",
}

defaultCommentMark = '#'
if sys.version_info[0] < 3:
    # OK this is a tad silly
    _lArrow = u'\u250C'.encode('utf-8')
    _rArrow = u'\u2514'.encode('utf-8')
    _vArrow = u'\u2502'.encode('utf-8')
    _hArrow = u'\u2500'.encode('utf-8')
else:
    _lArrow = u'\u250C'
    _rArrow = u'\u2514'
    _vArrow = u'\u2502'
    _hArrow = u'\u2500'


class Node():
    '''
    This is the main data structure of a dependency, a Node instance is a node
    on the tree. The structure of the subtree with node x as root can be viewed
    by calling x.__repr__()
    '''
    def __init__(self, parent=None):
        self.value = ()
        self.phrase = []
        self.id = 0
        self.parent = parent
        self.deprel = ""
        self.leftChild = None
        self.rightChild = None
        self.sibling = None
        self.depth = -1
        self.format = "Unspecified"
        return

    def __repr__(self, __spacing=[], __showSibling=False):
        '''
        This method prints the structure of the subtree with self as root.
        '''
        if self.leftChild is not None:
            self.leftChild.__repr__(__spacing + [_lArrow], True)

        last = _rArrow
        for i, entry in enumerate(__spacing):
            if i == 0:
                print(' ', end='')
            if i == len(__spacing) - 1:
                print(entry, end='')
            elif (__spacing[i + 1] == _rArrow and entry == _lArrow) or\
                    (__spacing[i + 1] == _lArrow and entry == _rArrow):
                print(_vArrow + "       ", end='')
            else:
                print("        ", end='')
        if self.parent is None:
            print("ROOT")
        elif len(__spacing) == 0:
            print(self.value[0])
        else:
            print(_hArrow + self.deprel + _hArrow + self.value[0])
        if self.rightChild is not None:
            self.rightChild.__repr__(__spacing + [_rArrow], True)

        if self.sibling is not None and __showSibling is True:
            self.sibling.__repr__(__spacing, True)
        return "\nRepresentation: " +\
            "conll.Node(\"" + str((self.id,) + self.value) + "\")\n" +\
            "Leafnode Label: " + str([n.value[0] for n in self.phrase]) +\
            "\n"

    def __len__(self):
        return len(self.phrase)

    def calcPhrase(self, force=False):
        if self.phrase == [] or force is True:
            self.phrase = []
            if self.leftChild is not None:
                self.phrase += self.leftChild.calcPhrase(force)

            if self.parent is not None:
                self.phrase += [self]

            if self.rightChild is not None:
                self.phrase += self.rightChild.calcPhrase(force)

            if self.sibling is not None:
                self.sibling.calcPhrase(force)

        if self.sibling is not None:
            return self.phrase + self.sibling.calcPhrase()
        return self.phrase

    def export(self):
        raise NotImplementedError


def constructFromText(rawContent, entryIndex=defaultEntryIndex):
    content = [line.strip().split('\t') for line in rawContent]
    # adding the root node
    nodes = [Node()]
    nodes[0].value = ("-ROOT-", )

    for i, line in enumerate(content, start=1):
        # Check ID for data integrity
        if int(line[entryIndex["ID"]]) != i:
            sys.stderr.write(
                "natlang.format.conll [WARN]: Corrupt data format\n")
            return None

        # force the first value in node.value to be FORM
        # temporarily store parent id in node.parent
        # store everything else in node.value
        newNode = Node()
        if "__name__" in entryIndex:
            newNode.format = entryIndex["__name__"]
        newNode.id = i
        newNode.parent = int(line[entryIndex["HEAD"]])
        newNode.deprel = line[entryIndex["DEPREL"]]

        newNode.value = (line[entryIndex["FORM"]], )
        for i, item in enumerate(line):
            if i != entryIndex["ID"] and i != entryIndex["HEAD"] and\
                    i != entryIndex["DEPREL"] and i != entryIndex["FORM"]:
                newNode.value += (line[i] if line[i] != '_' else None, )

        nodes.append(newNode)

    # replace node.parent with real entity.
    # add sibling, leftChild, rightChild
    for node in nodes[1:]:
        node.parent = nodes[node.parent]
        if node.parent.id > node.id:
            # leftChild
            if node.parent.leftChild is None:
                node.parent.leftChild = node
                continue
            tmp = node.parent.leftChild
            while tmp.sibling is not None:
                tmp = tmp.sibling
            tmp.sibling = node
        else:
            # rightChild
            if node.parent.rightChild is None:
                node.parent.rightChild = node
                continue
            tmp = node.parent.rightChild
            while tmp.sibling is not None:
                tmp = tmp.sibling
            tmp.sibling = node
    nodes[0].calcPhrase(force=True)
    return nodes[0]


def load(fileName,
         linesToLoad=sys.maxsize,
         entryIndex=defaultEntryIndex, commentMark=defaultCommentMark,
         verbose=False):
    fileName = os.path.expanduser(fileName)
    content = []
    widgets = [progressbar.Bar('>'), ' ', progressbar.ETA(),
               progressbar.FormatLabel(
               '; Total: %(value)d lines (in: %(elapsed)s)')]
    if verbose is False:
        loadProgressBar =\
            progressbar.ProgressBar(widgets=widgets,
                                    maxval=min(
                                        sum(1 for line in open(fileName)),
                                        linesToLoad)).start()
    i = 0
    entry = []
    with open(fileName) as file:
        for rawLine in file:
            i += 1
            if verbose is False:
                loadProgressBar.update(i)
            line = rawLine.strip()

            if line != "" and line[0] != commentMark:
                entry.append(line)
            else:
                content.append(constructFromText(entry))
                entry = []
                if i >= linesToLoad:
                    break

    if len(entry) > 0:
        content.append(constructFromText(entry))

    if verbose is False:
        loadProgressBar.finish()
    return content


class TestTree(unittest.TestCase):
    def testBuildTreeA(self, x=None):
        rawLine = [
            "1	From	from	ADP	IN	_	3	case	_	_",
            "2	the	the	DET	DT	Definite=Def|PronType=Art	3	det	_	_",
            "3	AP	AP	PROPN	NNP	Number=Sing	4	nmod	_	_",
            "4	comes	come	VERB	VBZ,	" +
            "Mood=Ind|Number=Sing|Person=3|Tense=Pres|VerbForm=Fin	0	root	_	_",
            "5	this	this	DET	DT	Number=Sing|PronType=Dem	6	det	_	_",
            "6	story	story	NOUN	NN	Number=Sing	4	nsubj	_	_",
            "7	:	:	PUNCT	:	_	4	punct	_	_]"]
        if x is None:
            x = constructFromText(rawLine)

        correctEntirePhraseForm = ["From", "the", "AP", "comes", "this",
                                   "story", ":"]
        correctLSubPhraseForm = ["From", "the", "AP"]
        correctRSubPhraseForm = ["this", "story"]

        self.assertSequenceEqual(
            correctEntirePhraseForm,
            [n.value[0] for n in x.phrase])
        self.assertSequenceEqual(
            correctLSubPhraseForm,
            [n.value[0] for n in x.rightChild.leftChild.phrase])
        self.assertSequenceEqual(
            correctRSubPhraseForm,
            [n.value[0] for n in x.rightChild.rightChild.phrase])
        return

    def testBuildTreeB(self, x=None):
        rawText = """1	President	President	PROPN	NNP	Number=Sing	2	compound	_	_
        2	Bush	Bush	PROPN	NNP	Number=Sing	5	nsubj	_	_
        3	on	on	ADP	IN	_	4	case	_	_
        4	Tuesday	Tuesday	PROPN	NNP	Number=Sing	5	nmod	_	_
        5	nominated	nominate	VERB	VBD	Mood=Ind|Tense=Past|VerbForm=Fin""" +\
            """	0	root	_	_
        6	two	two	NUM	CD	NumType=Card	7	nummod	_	_
        7	individuals	individual	NOUN	NNS	Number=Plur	5	dobj	_	_
        8	to	to	PART	TO	_	9	mark	_	_
        9	replace	replace	VERB	VB	VerbForm=Inf	5	advcl	_	_
        10	retiring	retire	VERB	VBG	VerbForm=Ger	11	amod	_	_
        11	jurists	jurist	NOUN	NNS	Number=Plur	9	dobj	_	_
        12	on	on	ADP	IN	_	14	case	_	_
        13	federal	federal	ADJ	JJ	Degree=Pos	14	amod	_	_
        14	courts	court	NOUN	NNS	Number=Plur	11	nmod	_	_
        15	in	in	ADP	IN	_	18	case	_	_
        16	the	the	DET	DT	Definite=Def|PronType=Art	18	det	_	_
        17	Washington	Washington	PROPN	NNP	Number=Sing	18	compound	_	_
        18	area	area	NOUN	NN	Number=Sing	14	nmod	_	SpaceAfter=No
        19	.	.	PUNCT	.	_	5	punct	_	_"""
        if x is None:
            x = constructFromText(rawText.split('\n'))

        correctEntirePhraseForm = [
            "President", "Bush", "on", "Tuesday", "nominated", "two",
            "individuals", "to", "replace", "retiring", "jurists", "on",
            "federal", "courts", "in", "the", "Washington", "area", "."]
        correctLSubPhraseForm = ["President", "Bush"]
        correctLSSubPhraseForm = ["on", "Tuesday"]
        correctRSubPhraseForm = ["two", "individuals"]
        correctRSSubPhraseForm = [
            "to", "replace", "retiring", "jurists", "on", "federal", "courts",
            "in", "the", "Washington", "area"]

        self.assertSequenceEqual(
            correctEntirePhraseForm,
            [n.value[0] for n in x.phrase])
        self.assertSequenceEqual(
            correctLSubPhraseForm,
            [n.value[0] for n in x.rightChild.leftChild.phrase])
        self.assertSequenceEqual(
            correctLSSubPhraseForm,
            [n.value[0] for n in x.rightChild.leftChild.sibling.phrase])
        self.assertSequenceEqual(
            correctRSSubPhraseForm,
            [n.value[0] for n in x.rightChild.rightChild.sibling.phrase])
        return

    def testLoader(self):
        currentdir = os.path.dirname(
            os.path.abspath(inspect.getfile(inspect.currentframe())))
        parentdir = os.path.dirname(currentdir)
        content = load(parentdir + "/test/sampleCoNLLU.conll", verbose=True)
        A = content[0]
        B = content[1]
        self.testBuildTreeA(A)
        self.testBuildTreeB(B)


if __name__ == '__main__':
    if not bool(getattr(sys, 'ps1', sys.flags.interactive)):
        unittest.main()
    else:
        rawLine = [
            "1	From	from	ADP	IN	_	3	case	_	_",
            "2	the	the	DET	DT	Definite=Def|PronType=Art	3	det	_	_",
            "3	AP	AP	PROPN	NNP	Number=Sing	4	nmod	_	_",
            "4	comes	come	VERB	VBZ,	" +
            "Mood=Ind|Number=Sing|Person=3|Tense=Pres|VerbForm=Fin	0	root	_	_",
            "5	this	this	DET	DT	Number=Sing|PronType=Dem	6	det	_	_",
            "6	story	story	NOUN	NN	Number=Sing	4	nsubj	_	_",
            "7	:	:	PUNCT	:	_	4	punct	_	_]"]
        x = constructFromText(rawLine)
        rawText = """1	President	President	PROPN	NNP	Number=Sing	2	compound	_	_
        2	Bush	Bush	PROPN	NNP	Number=Sing	5	nsubj	_	_
        3	on	on	ADP	IN	_	4	case	_	_
        4	Tuesday	Tuesday	PROPN	NNP	Number=Sing	5	nmod	_	_
        5	nominated	nominate	VERB	VBD	Mood=Ind|Tense=Past|VerbForm=Fin""" +\
            """	0	root	_	_
        6	two	two	NUM	CD	NumType=Card	7	nummod	_	_
        7	individuals	individual	NOUN	NNS	Number=Plur	5	dobj	_	_
        8	to	to	PART	TO	_	9	mark	_	_
        9	replace	replace	VERB	VB	VerbForm=Inf	5	advcl	_	_
        10	retiring	retire	VERB	VBG	VerbForm=Ger	11	amod	_	_
        11	jurists	jurist	NOUN	NNS	Number=Plur	9	dobj	_	_
        12	on	on	ADP	IN	_	14	case	_	_
        13	federal	federal	ADJ	JJ	Degree=Pos	14	amod	_	_
        14	courts	court	NOUN	NNS	Number=Plur	11	nmod	_	_
        15	in	in	ADP	IN	_	18	case	_	_
        16	the	the	DET	DT	Definite=Def|PronType=Art	18	det	_	_
        17	Washington	Washington	PROPN	NNP	Number=Sing	18	compound	_	_
        18	area	area	NOUN	NN	Number=Sing	14	nmod	_	SpaceAfter=No
        19	.	.	PUNCT	.	_	5	punct	_	_"""
        y = constructFromText(rawText.split('\n'))
        print("Use the two Nodes x and y for testing new methods on Node.")
        print("Use unittest.main() to start unit test")



# -*- coding: utf-8 -*-
# Python version: 2/3
#
# Text and Tree loader
# Simon Fraser University
# Jetic Gu
#
#
import os
import sys
from .tree import load as loadTree
from .txt import load as loadTxt
__version__ = "0.3a"


def load(file, linesToLoad=sys.maxsize):
    try:
        contents = loadTree(file, linesToLoad)
        if len([f for f in contents if f is not None]) < (len(contents) / 2):
            return loadTxt(file, linesToLoad)
    except AttributeError:
        return loadTxt(file, linesToLoad)
    return contents



# -*- coding: utf-8 -*-
# Python version: 2/3
#
# Jetic's file converter tool for NLP datasets
# Simon Fraser University
# Jetic Gu
#
# This module contains functions for loading and converting
# datasets in multiple formats.
#
import io
import os
import sys
import xml.etree.ElementTree as ET

import jieba

from .format.tree import lexicaliseNode
from .format.tree import load as loadPennTree


def procCoNaLa(filename,
               intent_output='intent.txt',
               snippet_output='snippets.txt'):

    import json
    import ast
    with io.open(filename, encoding='utf8') as f:
        data = json.load(f, encoding='utf8')

    skipped = 0
    with io.open(intent_output, 'w', encoding='utf8') as intent_file, \
            io.open(snippet_output, 'w', encoding='utf8') as snippet_file:
        for entry in data:
            rewritten_intent = entry['rewritten_intent']
            snippet = entry['snippet']
            if rewritten_intent is None:
                skipped += 1
            else:
                try:
                    ast.parse(snippet)
                except SyntaxError:
                    skipped += 1
                else:
                    x = rewritten_intent.replace(u'\r', u' ')
                    y = x.replace(u'\n', u' ')
                    intent_file.write(y)
                    intent_file.write(u'\n')
                    snippet_file.write(unicode(repr(snippet)))
                    snippet_file.write(u'\n')
    print('transformation complete. {} entries skipped.'.format(skipped))


def procXML(filename, jieba=False):
    result = []
    tree = ET.parse(filename)
    root = tree.getroot()

    for post in root:
        for su in post:
            result.append([su.text])
        result.append('')
    return result


def procXMLCN(filename):
    result = []
    tree = ET.parse(filename)
    root = tree.getroot()

    for post in root:
        for su in post:
            result.append(jieba.lcut(su.text))
        result.append('')
    return result


def rawIntoSegForms(fileName, linesToLoad=sys.maxsize):
    result = []
    fileName = os.path.expanduser(fileName)
    content = [line.strip() for line in open(fileName)][:linesToLoad]
    for line in content:
        result.append(jieba.lcut(line))
    return result


def pennTreeIntoTags(fileName, linesToLoad=sys.maxsize):
    result = []
    fileName = os.path.expanduser(fileName)
    content = [line.strip() for line in open(fileName)][:linesToLoad]
    for line in content:
        sentence = []
        procLine = line
        for ch in ('(', ')', '[', ']'):
            procLine = procLine.replace(ch, ' ')
        procLine = procLine.split()
        for i in range(len(procLine)):
            if procLine[i].isdigit():
                sentence.append(procLine[i - 1])
                print("Missing elements!")
        # sentence = [entry for entry in sentence if entry != "-NONE-"]
        result.append(sentence)
    return result


def pennTreeNoWords(fileName, linesToLoad=sys.maxsize):
    w2int = {"<UNK>": "XX"}
    t2int = {"<UNK>": "XX"}
    result = []
    fileName = os.path.expanduser(fileName)
    content = loadPennTree(fileName, linesToLoad)
    for node in content:
        sentence = lexicaliseNode(node, w2int, t2int).export()
        result.append((sentence,))
    return result


def tokenIntoForms(fileName, linesToLoad=sys.maxsize):
    result = []
    fileName = os.path.expanduser(fileName)
    content = [line.strip() for line in open(fileName)][:linesToLoad]
    for line in content:
        sentence = []
        procLine = line.split()
        for i in range(len(procLine)):
            sentence.append(procLine[i - 1].split(";")[-1])
        # sentence = [entry for entry in sentence if entry != "*"]
        sentence = [entry for entry in sentence]
        result.append(sentence)
    return result


def sgmIntoText(fileName, linesToLoad=sys.maxsize):
    result = []
    fileName = os.path.expanduser(fileName)
    content = [line.replace('>', '<').split("<")
               for line in open(fileName)][:linesToLoad]
    content = [[line[2]] for line in content if line[-2] == '/seg']
    return content


def removeEmptyLines(fileName, linesToLoad=sys.maxsize):
    result = []
    fileName = os.path.expanduser(fileName)
    content = [line.split() for line in open(fileName) if line.strip() != ""][
        :linesToLoad]
    return content


def rawIntoForms(fileName, linesToLoad=sys.maxsize):
    result = []
    fileName = os.path.expanduser(fileName)
    content = [line.strip() for line in open(fileName)][:linesToLoad]
    for line in content:
        sentence = []
        procLine = line.split()
        for i in range(len(procLine)):
            sentence.append(procLine[i].split(";")[-1])
        # sentence = [entry for entry in sentence if entry != "*"]
        sentence = [entry for entry in sentence]
        result.append(sentence)
    return result


def alignmentToList(fileName, linesToLoad=sys.maxsize):
    result = []
    fileName = os.path.expanduser(fileName)
    content = [line.strip() for line in open(fileName)][:linesToLoad]
    for line in content:
        result.append(line.split())
    return result


def pennTreeSplitIntoPennTree(fileName, linesToLoad=sys.maxsize):
    result = []
    fileName = os.path.expanduser(fileName)
    content = [line.strip() for line in open(fileName)][:linesToLoad]
    for line in content:
        result += line.replace("(", " ( ").replace(")", " ) ").split()
    results = []
    counter = 0
    for element in result:
        if counter == 0:
            results.append([])
            if element != "(":
                results[-1].append("(")
                counter += 1
                element = element[1:]
        results[-1].append(element)
        for char in element:
            if char == "(":
                counter += 1
            if char == ")":
                counter -= 1
    if counter != 0:
        raise RuntimeError
    return results


def armSplitIntoARMAndText(fileName, linesToLoad=sys.maxsize):
    result = []
    text = []
    fileName = os.path.expanduser(fileName)
    content = [line.strip() for line in open(fileName)][:linesToLoad]
    for line in content:
        if len(line) != 0 and line[0] == "#":
            if line[:8] == "# ::snt ":
                text.append(line[8:])
            continue
        result += line.replace("(", " ( ").replace(")", " ) ").split()
    results = []
    counter = 0
    for element in result:
        if counter == 0:
            results.append([])
            if element != "(":
                results[-1].append("(")
                counter += 1
                element = element[1:]
        results[-1].append(element)
        for char in element:
            if char == "(":
                counter += 1
            if char == ")":
                counter -= 1
    if counter != 0:
        raise RuntimeError
    return results, text


def convertFiles(filePattern, converter, output="o.pos"):
    """
    This function first scans the files matching the filePattern, then run each
    of them through the converter, the writes the results to output file(s).
    The output can be a list/tuple of str or simpe an str(single file).
    The converter's output can be a list (single output file) or a tuple of
    lists (multiple output files).
    Each converter's entry (a single line in each output file) can be either a
    list (which will be joined by space) or an str (one line, no "\n" at the
    end).
    """
    if isinstance(output, list) or isinstance(output, tuple):
        numFiles = len(output)
    else:
        numFiles = 1
        output = [output, ]
    result = [[] for item in output]
    import glob
    for name in glob.glob(os.path.expanduser(filePattern)):
        if os.path.isfile(name):
            print(name)
            converterOutput = converter(name)
            if not isinstance(converterOutput, tuple):
                converterOutput = (converterOutput,)
            if len(converterOutput) != numFiles:
                print(len(converterOutput))
                raise RuntimeError("Incorrect return entry length")
            for i in range(len(output)):
                result[i] += converterOutput[i]

    file = [open(os.path.expanduser(outputFile), "w") for outputFile in output]
    for i in range(numFiles):
        for sentence in result[i]:
            if isinstance(sentence, str):
                file[i].write(sentence)
            else:
                file[i].write(" ".join(sentence))
            file[i].write("\n")
    for f in file:
        f.close()
    return


def ASPECtoBitext(path, file1out, file2out, linesToLoad=sys.maxsize, pos=3):
    path = os.path.expanduser(path)
    bitext = list(open(path))[:linesToLoad]
    bitext = [entry.strip().split(" ||| ")[pos:] for entry in bitext]
    with open(os.path.expanduser(file1out), "w") as f1:
        with open(os.path.expanduser(file2out), "w") as f2:
            for entry in bitext:
                f1.write(entry[0])
                f2.write(entry[1])
                f1.write("\n")
                f2.write("\n")
    return


def alignedRawText(file1, file2, file1out, file2out, linesToLoad=sys.maxsize):
    path1 = os.path.expanduser(file1)
    path2 = os.path.expanduser(file2)
    bitext = list(zip(open(path1), open(path2)))[:linesToLoad]
    for i in range(len(bitext)):
        f, e = bitext[i]
        if f.strip() == "" or e.strip() == "":
            bitext[i] = None
    bitext = [entry for entry in bitext if entry is not None]
    with open(os.path.expanduser(file1out), "w") as f1:
        with open(os.path.expanduser(file2out), "w") as f2:
            for f, e in bitext:
                f1.write(f)
                f2.write(e)
    return


def alignedTextTree(textFile, treeFile, treeOut, linesToLoad=sys.maxsize):
    path1 = os.path.expanduser(textFile)
    path2 = os.path.expanduser(treeFile)

    text = list(open(path1))[:linesToLoad]
    tree = list(open(path2))[:linesToLoad]

    for i in range(len(text)):
        if text[i].strip() == "":
            tree.insert(i, "\n")

    with open(os.path.expanduser(treeOut), "w") as f1:
        for f in tree:
            f1.write(f)
    return


if __name__ == '__main__':
    """convertFiles(
        "/Users/jetic/Daten/align-data/eriguchi/train.de-en.rawtree.en",
        pennTreeSplitIntoPennTree,
        "/Users/jetic/Daten/align-data/eriguchi/train.de-en.tree.en")
    convertFiles(
        "/Users/jetic/Daten/align-data/eriguchi/train.cs-en.rawtree.en",
        pennTreeSplitIntoPennTree,
        "/Users/jetic/Daten/align-data/eriguchi/train.cs-en.tree.en")
    convertFiles(
        "/Users/jetic/Daten/align-data/eriguchi/train.ru-en.rawtree.en",
        pennTreeSplitIntoPennTree,
        "/Users/jetic/Daten/align-data/eriguchi/train.ru-en.tree.en")

    convertFiles(
        "/Users/jetic/Daten/align-data/eriguchi/dev.de-en.rawtree.en",
        pennTreeSplitIntoPennTree,
        "/Users/jetic/Daten/align-data/eriguchi/dev.de-en.tree.en")
    convertFiles(
        "/Users/jetic/Daten/align-data/eriguchi/dev.cs-en.rawtree.en",
        pennTreeSplitIntoPennTree,
        "/Users/jetic/Daten/align-data/eriguchi/dev.cs-en.tree.en")
    convertFiles(
        "/Users/jetic/Daten/align-data/eriguchi/dev.ru-en.rawtree.en",
        pennTreeSplitIntoPennTree,
        "/Users/jetic/Daten/align-data/eriguchi/dev.ru-en.tree.en")

    convertFiles(
        "/Users/jetic/Daten/align-data/eriguchi/test.de-en.rawtree.en",
        pennTreeSplitIntoPennTree,
        "/Users/jetic/Daten/align-data/eriguchi/test.de-en.tree.en")
    convertFiles(
        "/Users/jetic/Daten/align-data/eriguchi/test.cs-en.rawtree.en",
        pennTreeSplitIntoPennTree,
        "/Users/jetic/Daten/align-data/eriguchi/test.cs-en.tree.en")
    convertFiles(
        "/Users/jetic/Daten/align-data/eriguchi/test.ru-en.rawtree.en",
        pennTreeSplitIntoPennTree,
        "/Users/jetic/Daten/align-data/eriguchi/test.ru-en.tree.en")"""

    """alignedRawText(
        "~/Daten/align-data/eriguchi/train.de-en.tok.en",
        "~/Daten/align-data/eriguchi/train.de-en.tok.de",
        "~/Daten/align-data/eriguchi/train.de-en.clean.en",
        "~/Daten/align-data/eriguchi/train.de-en.clean.de")

    alignedRawText(
        "~/Daten/align-data/eriguchi/train.cs-en.tok.en",
        "~/Daten/align-data/eriguchi/train.cs-en.tok.cs",
        "~/Daten/align-data/eriguchi/train.cs-en.clean.en",
        "~/Daten/align-data/eriguchi/train.cs-en.clean.cs")

    alignedRawText(
        "~/Daten/align-data/eriguchi/train.ru-en.tok.en",
        "~/Daten/align-data/eriguchi/train.ru-en.tok.ru",
        "~/Daten/align-data/eriguchi/train.ru-en.clean.en",
        "~/Daten/align-data/eriguchi/train.ru-en.clean.ru")"""
    """ASPECtoBitext(
        "/Volumes/Schwarzbox/Sink/mark18/ASPEC/ASPEC-JE/train/train-1.txt",
        "train.raw.jp", "train.raw.en", 100000)

    ASPECtoBitext(
        "/Volumes/Schwarzbox/Sink/mark18/ASPEC/ASPEC-JE/dev/dev.txt",
        "dev.raw.jp", "dev.raw.en", 100000, 2)

    ASPECtoBitext(
        "/Volumes/Schwarzbox/Sink/mark18/ASPEC/ASPEC-JE/test/test.txt",
        "test.raw.jp", "test.raw.en", 100000, 2)

    convertFiles("/Users/jetic/Daten/semantic-data/amr_2.0/raw/data/amrs/split/training/amr-release-2.0-amrs-training-cctv.txt",
        armSplitIntoARMAndText,
        ("output.amr", "output.txt"))
    """



# run with python2 and python3 separately, to check whether the data is valid python2/3 syntax

from __future__ import print_function
import ast
import argparse


def check_syntax(code):
    try:
        ast.parse(code)
    except SyntaxError:
        valid = False
    else:
        valid = True
    return valid


if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='check syntax of python code')
    parser.add_argument('file')
    args = parser.parse_args()

    results = []
    with open(args.file) as f:
        for line in f:
            code = eval(line)
            valid = check_syntax(code)
            results.append(valid)

    print(repr(results))



import unittest
from . import AMR
from . import pyCode
from . import semanticFrame
from . import tree
from . import txt
from . import txtFiles
from . import txtOrTree
from . import conll


modules = {
    AMR,
    pyCode,
    semanticFrame,
    tree,
    txt,
    txtFiles,
    txtOrTree,
    conll
}


def testSuite():
    loader = unittest.TestLoader()
    suite = unittest.TestSuite()

    # add tests to the test suite
    for module in modules:
        suite.addTests(loader.loadTestsFromModule(module))
    return suite



# -*- coding: utf-8 -*-
# Python version: 2/3
#
# Python format Code (CoNaLa) loader
# Simon Fraser University
# Ruoyi Wang
#
#
import ast
import sys
import os
from .tree import Node


class _TmpNode:
    def __init__(self, tag, value):
        self.tag = tag
        self.value = value
        self.children = []

    def __repr__(self):
        return 'TmpNode({}, {})'.format(repr(self.tag), repr(self.value))


def _translate(py_ast):
    """translate python ast into custom class TmpNode"""
    ignore_list = ('lineno', 'col_offset', 'ctx')

    if isinstance(py_ast, _TmpNode):
        for i, child in enumerate(py_ast.children):
            py_ast.children[i] = _translate(child)
        return py_ast
    elif not isinstance(py_ast, ast.AST):
        # literal
        return _TmpNode('LITERAL', py_ast)
    else:
        node = _TmpNode(py_ast.__class__.__name__, None)
        for field, value in ast.iter_fields(py_ast):
            if field not in ignore_list:
                if isinstance(value, list):
                    # star-production
                    # this child is a list
                    # transform into a standalone node
                    vec_child = _TmpNode(field + '_vec', None)
                    vec_child.children = list(value)
                    node.children.append(vec_child)
                elif value is None:
                    # optional-production
                    vec_child = _TmpNode(field + '_optional', None)
                    node.children.append(vec_child)
                else:
                    node.children.append(value)

        for i, child in enumerate(node.children):
            node.children[i] = _translate(child)

        return node


def _restructure_rec(node, orig_children):
    """
    `node` is the already transformed node (type=tree.Node)
    `orig_children` is a list of the children corresponds to `node`
        (type=[TmpNode])
    """
    # edge case
    tag = node.value[0]
    if (tag.endswith('_vec') or tag.endswith('_optional')) and\
            not orig_children:
        # transformed grammar with no children
        dummy = Node()
        dummy.value = ('DUMMY', None)
        node.child = dummy
        dummy.parent = node

    # transform each child node
    child_nodes = []
    for orig_child in orig_children:
        child_node = Node()
        if orig_child.value is None:
            # internal node
            child_node.value = (orig_child.tag,)
        else:
            # leaf node
            child_node.value = (orig_child.tag, orig_child.value)
        child_nodes.append(child_node)

    # link child nodes
    for i, child_node in enumerate(child_nodes):
        child_node.parent = node
        if i == 0:
            node.child = child_node
        if i + 1 < len(child_nodes):
            # not last node
            child_node.sibling = child_nodes[i + 1]

    # recurse
    for child_node, orig_child in zip(child_nodes, orig_children):
        _restructure_rec(child_node, orig_child.children)


def _restructure(tmp_node):
    """transform the structure of TmpNode into Node"""
    node = Node()
    if tmp_node.value is None:
        node.value = (tmp_node.tag,)
    else:
        node.value = (tmp_node.tag, tmp_node.value)

    _restructure_rec(node, tmp_node.children)

    # append topmost root node
    root = Node()
    root.value = ('ROOT',)
    root.child = node
    node.parent = root
    return root


def python_to_tree(code):
    py_ast = ast.parse(code)
    root = _translate(py_ast)
    res_root = _restructure(root)
    res_root.calcId(1)
    res_root.calcPhrase(force=True)
    return res_root


def load(fileName, linesToLoad=sys.maxsize, verbose=False):
    import progressbar
    fileName = os.path.expanduser(fileName)
    content = []
    i = 0
    widgets = [progressbar.Bar('>'), ' ', progressbar.ETA(),
               progressbar.FormatLabel(
                   '; Total: %(value)d sents (in: %(elapsed)s)')]
    if verbose is False:
        loadProgressBar = \
            progressbar.ProgressBar(widgets=widgets,
                                    maxval=min(
                                        sum(1 for line in open(fileName)),
                                        linesToLoad)).start()
    for line in open(fileName):
        i += 1
        if verbose is False:
            loadProgressBar.update(i)
        code = eval(line)
        content.append(python_to_tree(code))
        if i == linesToLoad:
            break
    if verbose is False:
        loadProgressBar.finish()
    return content


if __name__ == '__main__':
    if not bool(getattr(sys, 'ps1', sys.flags.interactive)):
        pass
    else:
        # viz tools
        from graphviz import Graph
        import os
        import errno

        def draw_tmp_tree(root, name='tmp'):
            try:
                os.makedirs('figures')
            except OSError as exc:  # Guard against race condition
                if exc.errno != errno.EEXIST:
                    raise

            fname = 'figures/{}'.format(name + '.gv')
            g = Graph(format='png', filename=fname)

            fringe = [root]
            while fringe:
                node = fringe.pop()
                g.node(str(id(node)), repr(node))
                for child in node.children:
                    fringe.append(child)
                    g.node(str(id(child)), repr(node))
                    g.edge(str(id(node)), str(id(child)))

            return g.render()

        def repr_n(node):
            return 'Node{}'.format(repr(node.value))

        def draw_res_tree(root, name='res'):
            try:
                os.makedirs('figures')
            except OSError as exc:  # Guard against race condition
                if exc.errno != errno.EEXIST:
                    raise

            fname = 'figures/{}'.format(name + '.gv')
            g = Graph(format='png', filename=fname)

            fringe = [root]
            while fringe:
                node = fringe.pop()
                g.node(str(id(node)), repr_n(node))
                if node.child is not None:
                    child = node.child
                    fringe.append(child)
                    g.node(str(id(child)), repr_n(node))
                    g.edge(str(id(node)), str(id(child)), color='red')

                if node.sibling is not None:
                    sibling = node.sibling
                    fringe.append(sibling)
                    g.node(str(id(sibling)), repr_n(node))
                    g.edge(str(id(node)), str(id(sibling)), color='blue')

                if node.parent is not None:
                    g.edge(str(id(node)), str(id(node.parent)), color='green')

            return g.render()

        # example data structures
        code = r"os.path.abspath('mydir/myfile.txt')"
        py_ast = ast.parse(code)
        root = _translate(py_ast)
        res_root = _restructure(root)

        # draw_tmp_tree(root)
        # draw_res_tree(res_root)



# -*- coding: utf-8 -*-
# Python version: 2/3
#
# Constituency Tree class
# Simon Fraser University
# Jetic Gu
#
# This module contains functions and classes necessary for loading single lined
# penn treebank format sentences with constituency information. 2 examples are
# provided in TestTree.
#
import sys
import os
import unittest
import inspect
from copy import deepcopy


class Node:
    '''
    This is the main data structure of a tree, a Node instance is a node on the
    tree. The structure of the subtree with node x as root can be viewed by
    calling x.__repr__()
    '''
    def __init__(self, parent=None):
        self.value = ()
        self.phrase = []
        self.id = 0
        self.parent = parent
        self.sibling = None
        self.child = None
        self.depth = -1
        return

    def __repr__(self, __spacing="", __showSibling=False):
        '''
        This method prints the structure of the subtree with self as root.
        '''
        if self.child is not None:
            print(__spacing + str((self.id,) + self.value)[:-1])
            self.child.__repr__(__spacing + "  ", True)
            print(__spacing + ")")
        else:
            print(__spacing + str((self.id,) + self.value))
        if self.sibling is not None and __showSibling is True:
            self.sibling.__repr__(__spacing, True)
        return "\nRepresentation: " +\
            "Node(\"" + str((self.id,) + self.value) + "\")\n" +\
            "Leafnode Label: " + str(self.phrase) + "\n"

    def __len__(self):
        return len(self.phrase)

    def export(self):
        result = "("
        if self.child is None:
            result += str(self.value[0]) + " " + str(self.value[1]) + ")"
        else:
            result += str(self.value[0]) + " " + self.child.export() + ")"
        if self.sibling is not None:
            result += " " + self.sibling.export()
        return result

    def calcId(self, id):
        '''
        This method calculates the ids of all nodes in the subtree using
        breadth-first search.
        @param id: id of self, also the starting point
        '''
        self.id = id
        queue = [self]
        while len(queue) != 0:
            newQueue = []
            for node in queue:
                tmp = node.child
                if tmp is None and len(node.value) != 2:
                    node.value += node.value
                    if len(node.value) != 2:
                        raise RuntimeError
                if tmp is not None and len(node.value) != 1:
                    raise RuntimeError
                if tmp is not None and "-" in node.value[0]:
                    newClauseType = node.value[0].split("-")[0]
                    node.value = (newClauseType,)
                while tmp is not None:
                    newQueue.append(tmp)
                    tmp.id = id + 1
                    id += 1
                    tmp = tmp.sibling
            queue = newQueue
        return

    def calcPhrase(self, force=False):
        if self.phrase == [] or self.depth <= 0 or force is True:
            self.phrase = []
            self.depth = 1
            if self.child is not None:
                self.child.calcPhrase(force)
                self.depth = max(self.depth, self.child.depth + 1)
            if self.sibling is not None:
                self.sibling.calcPhrase(force)
                self.depth = max(self.depth, self.sibling.depth)
        if self.child is None:
            self.phrase = [self.value]
        else:
            tmp = self.child
            while tmp is not None:
                self.phrase += tmp.phrase
                tmp = tmp.sibling
        return

    def columnFormat(self, parColumn=None, sibColumn=None,
                     valColumn=None, hasChild=None, hasSibl=None, LM=False):
        '''
        This method returns the fraternal info and ancestral info columns.
        @return anc, fra: lists
            anc[i] is the id of ancester of node i,
            fra[i] is the id of previous sibling of node i,
            val[i] is the label of node i,
        '''
        if parColumn is None or sibColumn is None or valColumn is None or\
                hasChild is None or hasSibl is None:
            parColumn = []
            sibColumn = []
            valColumn = []
            hasChild = []
            hasSibl = []
        while len(parColumn) <= self.id:
            parColumn.append(0)
            sibColumn.append(0)
            hasChild.append(0)
            hasSibl.append(0)
            valColumn.append(("NULL",))
        valColumn[self.id] = self.value

        if self.parent is not None:
            parColumn[self.id] = self.parent.id
            if self.parent.child != self:
                sibColumn[self.id] = self.id - 1
            else:
                if LM is True:
                    sibColumn[self.id] = sibColumn[self.parent.id]

        if self.child is not None:
            hasChild[self.id] = 1
            self.child.columnFormat(
                parColumn, sibColumn, valColumn, hasChild, hasSibl, LM)
        if self.sibling is not None:
            hasSibl[self.id] = 1
            self.sibling.columnFormat(
                parColumn, sibColumn, valColumn, hasChild, hasSibl, LM)
        return parColumn, sibColumn, valColumn, hasChild, hasSibl

    def columnFormatWordIndex(self, column=None, start=0):
        if column is None:
            column = []
        while len(column) <= self.id:
            column.append(0)
        column[self.id] = start
        if self.child is not None:
            self.child.columnFormatWordIndex(column, start=start)
        if self.sibling is not None:
            self.sibling.columnFormatWordIndex(column, start=start + len(self))
        return column


def constructTreeFromStr(string, rootLabel="ROOT"):
    '''
    This method constructs a tree from a string.
    @param string: str, in Penn Treebank format
    @return root: Node, the root node.
    '''
    if string.strip() == "(())":
        return None
    newString = string.replace("(", " ( ").replace(")", " ) ")
    try:
        return constructTree(newString.split(), rootLabel)
    except AttributeError as e:
        return None


def constructTree(elements, rootLabel="ROOT"):
    '''
    This method constructs a tree from a list of elements. Each bracket is
    considered an independent element.
    @param elements: list of str, in Penn Treebank format
    @return root: Node, the root node.
    '''
    root = None
    currentParent = None
    current = None
    for element in elements:
        if element == "(":
            currentParent = current
            current = Node(parent=currentParent)
            if currentParent is not None:
                if currentParent.child is not None:
                    tmp = currentParent.child
                    while tmp.sibling is not None:
                        tmp = tmp.sibling
                    tmp.sibling = current
                else:
                    currentParent.child = current
            else:
                root = current

        elif element == ")":
            current = current.parent
            if current is not None:
                currentParent = current.parent
        else:
            current.value += (element,)
    if root is not None:
        if root.value == ():
            root.value = (rootLabel,)
        try:
            root.calcId(1)
        except RuntimeError:
            return None
        root.calcPhrase(force=True)
    return root


def constructTreeFromRNNGAction(actions):
    root = None
    current = root
    openClauses = 0
    for action in actions:
        if action[0] == "GEN":
            if root is None:
                root = Node()
                root.value = ("NaN", "NaN")
                return root
            if openClauses == 1:
                openClauses = 0
                tmp = Node(current)
                current.child = tmp
            else:
                tmp = Node(current.parent)
                current.sibling = tmp
            tmp.value = ('NaN', action[1])
            current = tmp
        elif action[0] == "NT":
            if root is None:
                tmp = Node()
                root = tmp
            elif openClauses == 1:
                tmp = Node(current)
                current.child = tmp
            else:
                tmp = Node(current.parent)
                current.sibling = tmp
            tmp.value = (action[1],)
            current = tmp
            openClauses = 1
        else:
            if root is None:
                root = Node()
                root.value = ("NaN", "NaN")
                return root
            current = current.parent

    root.calcId(1)
    root.calcPhrase(force=True)
    return root


def load(fileName, linesToLoad=sys.maxsize, verbose=False):
    import progressbar
    fileName = os.path.expanduser(fileName)
    content = []
    i = 0
    widgets = [progressbar.Bar('>'), ' ', progressbar.ETA(),
               progressbar.FormatLabel(
               '; Total: %(value)d sents (in: %(elapsed)s)')]
    with open(fileName) as file:
        if verbose is False:
            loadProgressBar =\
                progressbar.ProgressBar(widgets=widgets,
                                        maxval=min(
                                            sum(1 for line in file),
                                            linesToLoad)).start()
        for line in file:
            i += 1
            if verbose is False:
                loadProgressBar.update(i)
            content.append(constructTreeFromStr(line))
            if i == linesToLoad:
                break

    if verbose is False:
        loadProgressBar.finish()
    return content


def lexicaliseNode(root, wLex, tLex=None, lLex=None):
    '''
    @param wLex: dist, word lexicon
    @param tLex: dist, postag lexicon
    @param lLex: dist, label lexicon
    '''
    def value2int(node):
        if node.child is None:
            if node.value[1] in wLex:
                node.value = (node.value[0], wLex[node.value[1]])
            else:
                node.value = (node.value[0], wLex["<UNK>"])
            if tLex is not None:
                if node.value[0] in tLex:
                    node.value = (tLex[node.value[0]], node.value[1])
                else:
                    node.value = (tLex["<UNK>"], node.value[1])
        else:
            if lLex is not None:
                if node.value[0] in lLex:
                    node.value = (lLex[node.value[0]],)
                else:
                    node.value = (lLex["<UNK>"],)
        return node

    newRoot = deepcopy(root)
    value2int(newRoot)
    queue = [newRoot]
    while len(queue) != 0:
        newQueue = []
        for node in queue:
            tmp = node.child
            while tmp is not None:
                newQueue.append(tmp)
                value2int(tmp)
                tmp = tmp.sibling
        queue = newQueue
    newRoot.calcPhrase(force=True)
    return newRoot


def constructRNNGAction(root):
    result = []
    if root.child is None:
        result += [("GEN", root.value[1])]
    else:
        result += [("NT", root.value[0])]
        result += constructRNNGAction(root.child)
        result += [("REDUCE",)]
    if root.sibling is not None:
        result += constructRNNGAction(root.sibling)
    return result


class TestTree(unittest.TestCase):
    def testBuildTreeA(self, x=None):
        elements = ["(", "ROOT",
                    "(", "S",
                    "(", "NP", "Andrei", ")",
                    "(", "VP", "(", "VP", "likes", ")",
                    "(", "NP", "cheese", ")",
                    ")",
                    ")",
                    ")"]
        if x is None:
            x = constructTree(elements)
        # print x.__repr__()
        anc, fra, val, child, sibl = x.columnFormat()
        ancG = [0, 0, 1, 2, 2, 4, 4]
        fraG = [0, 0, 0, 0, 3, 0, 5]
        valG = [('NULL',), ('ROOT',), ('S',), ('NP', 'Andrei'), ('VP',),
                ('VP', 'likes'), ('NP', 'cheese')]
        self.assertSequenceEqual(anc, ancG)
        self.assertSequenceEqual(fra, fraG)
        self.assertSequenceEqual(val, valG)
        return

    def testBuildTreeB(self, x=None):
        elements = "( ROOT ( FRAG ( NP ( NNP Madam ) ( NNP President ) )" +\
            " ( , , ) ( PP ( IN on ) ( NP ( NP ( DT a ) ( NN point ) ) (" +\
            " PP ( IN of ) ( NP ( NN order ) ) ) ) ) ( . . ) ) )"
        if x is None:
            x = constructTreeFromStr(elements)
        # print x.__repr__()
        anc, fra, val, child, sibl = x.columnFormat()
        ancG = [0, 0, 1, 2, 2, 2, 2, 3, 3, 5, 5, 10, 10, 11, 11, 12, 12, 16]
        fraG = [0, 0, 0, 0, 3, 4, 5, 0, 7, 0, 9, 0, 11, 0, 13, 0, 15, 0]
        valG = [('NULL',), ('ROOT',), ('FRAG',), ('NP',), (',', ','), ('PP',),
                ('.', '.'), ('NNP', 'Madam'), ('NNP', 'President'),
                ('IN', 'on'), ('NP',), ('NP',), ('PP',), ('DT', 'a'),
                ('NN', 'point'), ('IN', 'of'), ('NP',), ('NN', 'order')]
        self.assertSequenceEqual(anc, ancG)
        self.assertSequenceEqual(fra, fraG)
        self.assertSequenceEqual(val, valG)
        return

    def testLoadTreeFromFile(self):
        currentdir = os.path.dirname(
            os.path.abspath(inspect.getfile(inspect.currentframe())))
        parentdir = os.path.dirname(currentdir)
        content = load(parentdir + "/test/sampleTree.txt", verbose=True)
        A = content[0]
        B = content[1]
        self.testBuildTreeA(A)
        self.testBuildTreeB(B)
        return

    def testLexicaliseNode(self):
        elements = ["(", "ROOT",
                    "(", "S",
                    "(", "NP", "Andrei", ")",
                    "(", "VP", "(", "VP", "likes", ")",
                    "(", "NP", "cheese", ")",
                    ")",
                    ")",
                    ")"]
        x = constructTree(elements)
        w2int = {'<UNK>': 0, 'likes': 1, 'cheese': 2, 'Andrei': 3}
        y = lexicaliseNode(x, w2int)
        # print y.__repr__()
        self.assertSequenceEqual(y.phrase, [('NP', 3), ('VP', 1), ('NP', 2)])

        int2w = {0: '<UNK>', 1: 'likes', 2: 'cheese', 3: 'Andrei'}
        z = lexicaliseNode(y, int2w)
        # print y.__repr__()
        self.assertSequenceEqual(
            z.phrase, [('NP', "Andrei"), ('VP', "likes"), ('NP', "cheese")])
        return

    def testExportNode(self):
        elements = ["(", "ROOT",
                    "(", "S",
                    "(", "NP", "Andrei", ")",
                    "(", "VP", "(", "VP", "likes", ")",
                    "(", "NP", "cheese", ")",
                    ")",
                    ")",
                    ")"]
        x = constructTree(elements)
        self.assertEqual(elements, x.export().replace(
            "(", " ( ").replace(")", " ) ").split())
        return

    def testBuildTreeLM(self, x=None):
        elements = ["(", "ROOT",
                    "(", "S",
                    "(", "NP", "Andrei", ")",
                    "(", "VP", "(", "VP", "likes", ")",
                    "(", "NP", "cheese", ")",
                    ")",
                    ")",
                    ")"]
        if x is None:
            x = constructTree(elements)
        # print x.__repr__()
        anc, fra, val, child, sibl = x.columnFormat(LM=True)
        ancG = [0, 0, 1, 2, 2, 4, 4]
        fraG = [0, 0, 0, 0, 3, 3, 5]
        valG = [('NULL',), ('ROOT',), ('S',), ('NP', 'Andrei'), ('VP',),
                ('VP', 'likes'), ('NP', 'cheese')]
        self.assertSequenceEqual(anc, ancG)
        self.assertSequenceEqual(fra, fraG)
        self.assertSequenceEqual(val, valG)
        return

    def testColumnFormatWordIndex(self, x=None):
        elements = ["(", "ROOT",
                    "(", "S",
                    "(", "NP", "Andrei", ")",
                    "(", "VP", "(", "VP", "likes", ")",
                    "(", "NP", "cheese", ")",
                    ")",
                    ")",
                    ")"]
        if x is None:
            x = constructTree(elements)
        realColumn = [0, 0, 0, 0, 1, 1, 2]
        self.assertSequenceEqual(realColumn, x.columnFormatWordIndex())


if __name__ == '__main__':
    if not bool(getattr(sys, 'ps1', sys.flags.interactive)):
        unittest.main()
    else:
        elements = ["(", "ROOT",
                    "(", "S",
                    "(", "NP", "Andrei", ")",
                    "(", "VP", "(", "VP", "likes", ")",
                    "(", "NP", "cheese", ")",
                    ")",
                    ")",
                    ")"]
        x = constructTree(elements)
        elements = "( ROOT ( FRAG ( NP ( NNP Madam ) ( NNP President ) )" +\
            " ( , , ) ( PP ( IN on ) ( NP ( NP ( DT a ) ( NN point ) ) (" +\
            " PP ( IN of ) ( NP ( NN order ) ) ) ) ) ( . . ) ) )"
        y = constructTreeFromStr(elements)
        print("Use the two Nodes x and y for testing new methods on Node.")
        print("Use unittest.main() to start unit test")



# -*- coding: utf-8 -*-
# Python version: 2/3
#
# Text loader
# Simon Fraser University
# Jetic Gu
#
#
import os
import sys
__version__ = "0.3a"


def load(file, linesToLoad=sys.maxsize):
    return [line.strip().split() for line in open(os.path.expanduser(file))][
        :linesToLoad]



# -*- coding: utf-8 -*-
# Python version: 2/3
#
# Setup
# Simon Fraser University
# Jetic Gu
#
import setuptools
from natlang import version

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="natlang",
    version=version,
    author="Jetic Gū, Rory Wang",
    author_email="jeticg@sfu.ca",
    description="Natural language data loading tools",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/jeticg/datatool",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
          'jieba',
          'progressbar'
    ],
    test_suite='natlang.format.testSuite'
)



from subprocess import Popen, PIPE
import argparse


def check_file_py2(fpath, checker_path='syntax_checker', python_path='python2'):
    process = Popen([python_path, checker_path, fpath], stdout=PIPE)
    output, err = process.communicate()
    exit_code = process.wait()
    if exit_code == 0:
        return eval(output)
    else:
        raise RuntimeError(err)


def check_file_py3(fpath, checker_path='syntax_checker', python_path='python3'):
    process = Popen([python_path, checker_path, fpath], stdout=PIPE)
    output, err = process.communicate()
    exit_code = process.wait()
    if exit_code == 0:
        return eval(output)
    else:
        raise RuntimeError(err)


if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='check syntax of python code')
    parser.add_argument('code_file')
    args = parser.parse_args()

    py2_results = check_file_py2(args.code_file)
    py3_results = check_file_py3(args.code_file)
